#ifndef _MAPLE_EXTERNAL_C_H_
#define _MAPLE_EXTERNAL_C_H_

#include "mplshlib.h"

/*
**      Windows DLL Magic
*/
/* These are used to provide special annotations
   of the .dll exports on Windows. */
#if !defined(EXT_DECL)
#  ifdef _MSC_VER
#    ifdef WMI_WINNT
#      define EXT_DECL __declspec(dllexport)
#    else
#      define EXT_DECL __declspec(dllimport)
#    endif /* WMI_WINNT */
#  else
#    define EXT_DECL
#  endif /* _MSC_VER */
#endif /* EXT_DECL */

#if !defined(M_DECL)
#  ifdef _MSC_VER
#    define M_DECL __stdcall
#  else
#    define M_DECL
#  endif /* _MSC_VER */
#endif /* M_DECL */

#ifdef __cplusplus
extern "C" {
#endif

/* ---------------------------------------------------------------------- */
/*                       Memory Management                                */
/* ---------------------------------------------------------------------- */

/* Allocate Using Maple's Allocator */
EXT_DECL void* M_DECL MapleAlloc( MKernelVector kv, M_INT nbytes );

/* Free Memory Allocated By Maple's Allocator */ 
EXT_DECL void M_DECL MapleDispose( MKernelVector kv, ALGEB s );

/* Allow a to be garbage collected */
EXT_DECL void M_DECL MapleGcAllow( MKernelVector kv, ALGEB a );

/* Prevent a from being garbage collected */
EXT_DECL void M_DECL MapleGcProtect( MKernelVector kv, ALGEB a );

/* ---------------------------------------------------------------------- */
/*                       Assign to Maple Variables                        */
/* ---------------------------------------------------------------------- */

EXT_DECL ALGEB M_DECL MapleAssign( MKernelVector kv, ALGEB lhs, ALGEB rhs );

EXT_DECL ALGEB M_DECL MapleAssignIndexed( MKernelVector kv, ALGEB lhs, 
			M_INT dim, M_INT *ind, ALGEB rhs );


/* ---------------------------------------------------------------------- */
/*             Error Handling and User Information                        */
/* ---------------------------------------------------------------------- */

/* Raising Errors */
EXT_DECL void M_DECL MapleRaiseError( MKernelVector kv, char *msg );
EXT_DECL void M_DECL MapleRaiseError1( MKernelVector kv, char *msg, ALGEB arg1 );
EXT_DECL void M_DECL MapleRaiseError2( MKernelVector kv, char *msg, ALGEB arg1, ALGEB arg2 );

/* Evaluate a C Function, Trapping Any Raised Errors */
EXT_DECL void* M_DECL MapleTrapError( MKernelVector kv, 
      void *(*proc) ( void *data ), void *data, M_BOOL *errorflag );

/* Provide Run-time Information */
EXT_DECL void M_DECL MapleUserInfo( MKernelVector kv, int level, char *name, 
                         char *msg );   

/* output to Maple's output stream;
   setup by calling InitMaplePrintf, do printing by calling MaplePrintf
*/
EXT_DECL void M_DECL InitMaplePrintf( MKernelVector kv );
typedef int (M_DECL *tMaplePrintf) ( const char *, ... );
extern EXT_DECL tMaplePrintf MaplePrintf;


/* ---------------------------------------------------------------------- */
/*        Evaluate a Maple Procedure or DAG using hardware floats         */
/* ---------------------------------------------------------------------- */

EXT_DECL double M_DECL MapleEvalhf( MKernelVector kv, ALGEB s );

/* first argument here is args[1] */
EXT_DECL double M_DECL EvalhfMapleProc( MKernelVector kv, ALGEB fn, 
                            int nargs, double *args );

/* ---------------------------------------------------------------------- */
/*                 Evaluate a Maple Procedure or statement                */
/* ---------------------------------------------------------------------- */

EXT_DECL ALGEB M_DECL EvalMapleProc( MKernelVector kv, ALGEB fn, int nargs, 
                /* ALGEB arg1, ALGEB arg2, */ ... );

EXT_DECL ALGEB M_DECL EvalMapleStatement( MKernelVector kv, char *statement );

EXT_DECL ALGEB M_DECL MapleEval( MKernelVector kv, ALGEB s );

/* ---------------------------------------------------------------------- */
/*                          Data Queries                                  */
/* ---------------------------------------------------------------------- */

EXT_DECL M_BOOL M_DECL IsMapleAssignedName( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleComplexNumeric( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleNumeric( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleInteger( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleInteger8( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleInteger16( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleInteger32( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleInteger64( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleName( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleNULL( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMaplePointer( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMaplePointerNULL( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleProcedure( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleRTable( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleString( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleTable( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleUnassignedName( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL IsMapleUnnamedZero( MKernelVector kv, ALGEB s );

/* ---------------------------------------------------------------------- */
/*                             List Manipulation                          */
/* ---------------------------------------------------------------------- */

EXT_DECL ALGEB M_DECL MapleListAlloc( MKernelVector kv, M_INT num_members );

EXT_DECL void M_DECL MapleListAssign( MKernelVector kv, ALGEB list, M_INT i, 
                            ALGEB val );

EXT_DECL ALGEB M_DECL MapleListSelect( MKernelVector kv, ALGEB list, M_INT i );


/* ---------------------------------------------------------------------- */
/*                     Conversion from Maple Objects                      */
/* ---------------------------------------------------------------------- */

EXT_DECL COMPLEXF32 M_DECL MapleToComplexFloat32( MKernelVector kv, ALGEB s ); 

EXT_DECL COMPLEXF64 M_DECL MapleToComplexFloat64( MKernelVector kv, ALGEB s ); 

EXT_DECL CXDAG M_DECL MapleToComplexFloatDAG( MKernelVector kv, ALGEB s );

EXT_DECL FLOAT32 M_DECL MapleToFloat32( MKernelVector kv, ALGEB s );

EXT_DECL FLOAT64 M_DECL MapleToFloat64( MKernelVector kv, ALGEB s );

EXT_DECL INTEGER8 M_DECL MapleToInteger8( MKernelVector kv, ALGEB s );

EXT_DECL INTEGER16 M_DECL MapleToInteger16( MKernelVector kv, ALGEB s );

EXT_DECL INTEGER32 M_DECL MapleToInteger32( MKernelVector kv, ALGEB s );

EXT_DECL INTEGER64 M_DECL MapleToInteger64( MKernelVector kv, ALGEB s );

EXT_DECL M_BOOL M_DECL MapleToM_BOOL( MKernelVector kv, ALGEB s );

EXT_DECL M_INT M_DECL MapleToM_INT( MKernelVector kv, ALGEB s );

EXT_DECL void* M_DECL MapleToPointer( MKernelVector kv, ALGEB s );

EXT_DECL char* M_DECL MapleToString( MKernelVector kv, ALGEB s );

/* Determine the number of arguments in a Maple object */
EXT_DECL M_INT M_DECL MapleNumArgs( MKernelVector kv, ALGEB expr );

/* ---------------------------------------------------------------------- */
/*       Rectangular Table (vector, matrix, array) Manipulation           */
/* ---------------------------------------------------------------------- */

EXT_DECL void M_DECL RTableAppendAttribute( MKernelVector kv, RTableSettings *s, char *name );

EXT_DECL void M_DECL RTableAppendIndFn( MKernelVector kv, RTableSettings *s, 
                           ALGEB indfn );

EXT_DECL RTableData M_DECL RTableAssign( MKernelVector kv, ALGEB rt, 
                 M_INT *index, RTableData val );

EXT_DECL ALGEB M_DECL RTableCopy( MKernelVector kv, RTableSettings *s, 
                            ALGEB rt );

EXT_DECL ALGEB M_DECL RTableCopyImPart( MKernelVector kv, RTableSettings *s, 
                       ALGEB rt );

EXT_DECL ALGEB M_DECL RTableCopyRealPart( MKernelVector kv, RTableSettings *s, 
                              ALGEB rt );

EXT_DECL ALGEB M_DECL RTableCreate( MKernelVector kv, RTableSettings *s, 
                         void *pdata, M_INT *bounds );

EXT_DECL void* M_DECL RTableDataBlock( MKernelVector kv, ALGEB rt );

EXT_DECL void M_DECL RTableGetDefaults( MKernelVector kv, RTableSettings *s );

EXT_DECL void M_DECL RTableGetSettings( MKernelVector kv, RTableSettings *s, 
                           ALGEB rt );

EXT_DECL M_INT M_DECL RTableIndFn( MKernelVector kv, ALGEB rt, M_INT num );

EXT_DECL ALGEB M_DECL RTableIndFnArgs( MKernelVector kv, ALGEB rt, M_INT num );

EXT_DECL M_BOOL M_DECL RTableIsReal( MKernelVector kv, ALGEB rt );

EXT_DECL M_INT M_DECL RTableLowerBound( MKernelVector kv, ALGEB rt, M_INT dim );

EXT_DECL M_INT M_DECL RTableNumElements( MKernelVector kv, ALGEB rt );

EXT_DECL M_INT M_DECL RTableNumDimensions( MKernelVector kv, ALGEB rt );

EXT_DECL RTableData M_DECL RTableSelect( MKernelVector kv, ALGEB rt, 
                 M_INT *index );

EXT_DECL void M_DECL RTableSetAttribute( MKernelVector kv, RTableSettings *s, 
                 char *name );

EXT_DECL void M_DECL RTableSetIndFn( MKernelVector kv, RTableSettings *s, 
                ALGEB indfn );

EXT_DECL void M_DECL RTableSetType( MKernelVector kv, RTableSettings *s, 
                 M_INT id, char *name );

EXT_DECL void M_DECL RTableSparseCompact( MKernelVector kv, ALGEB rt );

EXT_DECL NAG_INT* M_DECL RTableSparseIndexRow( MKernelVector kv, ALGEB rt, 
                M_INT dim );

EXT_DECL ALGEB M_DECL RTableSparseIndexSort( MKernelVector kv, ALGEB rt, 
                M_INT by_dim );

EXT_DECL void M_DECL RTableSparseSetNumElems( MKernelVector kv, ALGEB rt, 
                M_INT num ); 

EXT_DECL M_INT M_DECL RTableSparseSize( MKernelVector kv, ALGEB rt );

EXT_DECL M_INT M_DECL RTableUpperBound( MKernelVector kv, ALGEB rt, M_INT dim );

EXT_DECL ALGEB M_DECL RTableZipReIm( MKernelVector kv, RTableSettings *s, 
                            ALGEB rt_re, ALGEB rt_im );

/* Data Selection */
EXT_DECL ALGEB M_DECL MapleSelectImaginaryPart( MKernelVector kv, ALGEB s );

EXT_DECL ALGEB M_DECL MapleSelectIndexed( MKernelVector kv, ALGEB s, M_INT dim, M_INT *ind );

EXT_DECL ALGEB M_DECL MapleSelectRealPart( MKernelVector kv, ALGEB s );

/* Data Uniquification */
EXT_DECL ALGEB M_DECL MapleUnique( MKernelVector kv,ALGEB s );

/* ---------------------------------------------------------------------- */
/*                             Table Manipulation                         */
/* ---------------------------------------------------------------------- */

EXT_DECL ALGEB M_DECL MapleTableAlloc( MKernelVector kv );

EXT_DECL void M_DECL MapleTableAssign( MKernelVector kv, ALGEB table, ALGEB ind, ALGEB val );

EXT_DECL void M_DECL MapleTableDelete( MKernelVector kv, ALGEB table, ALGEB ind );

EXT_DECL M_BOOL M_DECL MapleTableHasEntry( MKernelVector kv, ALGEB table, ALGEB ind );

EXT_DECL ALGEB M_DECL MapleTableSelect( MKernelVector kv, ALGEB table, ALGEB ind );

/* ---------------------------------------------------------------------- */
/*                        Conversion to Maple Objects                     */
/* ---------------------------------------------------------------------- */

EXT_DECL ALGEB M_DECL ToMapleBoolean( MKernelVector kv, long b );

EXT_DECL ALGEB M_DECL ToMapleChar( MKernelVector kv, long c );

EXT_DECL ALGEB M_DECL ToMapleComplex( MKernelVector kv, double re, double im );

EXT_DECL ALGEB M_DECL ToMapleComplexFloat( MKernelVector kv, ALGEB re, ALGEB im );

EXT_DECL ALGEB M_DECL ToMapleExpressionSequence( MKernelVector kv, int nargs, /* ALGEB arg1, ALGEB arg2, */ ... );

EXT_DECL ALGEB M_DECL ToMapleInteger( MKernelVector kv, long i ); 

EXT_DECL ALGEB M_DECL ToMapleInteger64( MKernelVector kv, INTEGER64 i ); 

EXT_DECL ALGEB M_DECL ToMapleFloat( MKernelVector kv, double f ); 

EXT_DECL ALGEB M_DECL ToMapleName( MKernelVector kv, char *n, M_BOOL is_global ); 

EXT_DECL ALGEB M_DECL ToMapleNULL( MKernelVector kv ); 

EXT_DECL ALGEB M_DECL ToMapleNULLPointer( MKernelVector kv ); 

EXT_DECL ALGEB M_DECL ToMaplePointer( MKernelVector kv, void *v );

EXT_DECL ALGEB M_DECL ToMapleRelation( MKernelVector kv, const char *rel, 
                          ALGEB lhs, ALGEB rhs );

EXT_DECL ALGEB M_DECL ToMapleString( MKernelVector kv, char *s ); 

EXT_DECL ALGEB M_DECL ToMapleUneval( MKernelVector kv, ALGEB s ); 

#ifdef __cplusplus
}
#endif

#endif /* _MAPLE_EXTERNAL_C_H_ */
