/* Maple Shared Library Header File
   Copyright (c) 1996 by Waterloo Maple, Inc.
   All rights reserved. Unauthorized duplication prohibited.
   Written October 1996 by Stefan Vorkoetter.

   *** THIS FILE IS INTENDED FOR DISTRIBUTION WITH THE MAPLE PRODUCT ***

   This file provides definitions required to implement shared modules that
   can be linked into a running Maple kernel. Typically, such modules are
   generated automatically by Maple's define_external function, and are then
   used to call other non-Maple shared modules. However, a user could, in
   principle, write a Maple-specific shared module that doesn't require this
   layering. */

#ifndef __MPLSHLIB_H__
#define __MPLSHLIB_H__

#undef M_DECL
#if defined WMI_WINNT || defined WMI_DOS
# define M_DECL __stdcall
# ifndef USE_PROTO
#  define USE_PROTO              /* stdcall requires prototypes */
# endif
#else
# define M_DECL
#endif

/* Decide whether or not to use ANSI C function prototypes. */
#if defined(__STDC__) || defined(__cplusplus) || defined(USE_PROTO)
# define MPL_P(x) x
#else
# define MPL_P(x) ()
#endif

#ifndef M_INT   /* Exclude most of this file when compiling Maple itself. */

/* Determine machine word size dependent parameters. */
#if defined __alpha /* 64-bit DEC-ALPHA Architecture */
# define M_INT long
# define RTABLE_LONG RTABLE_INTEGER64
# define WORDSIZE 64
#elif defined __sparcv9 /* 64-bit Solaris Architecture */
# define M_INT long
# define RTABLE_LONG RTABLE_INTEGER64
# define WORDSIZE 64
#else /* 32-bit Architectures */
# define M_INT int
# define RTABLE_LONG RTABLE_INTEGER32
# define WORDSIZE 32
#endif

/* Maple Internal Types */
typedef M_INT ***ALGEB;
typedef M_INT M_BOOL;
#define NAG_INT long
#define DAG(x) ((ALGEB)(x))
#define TRUE 1
#define FALSE 0
#define FAIL -1

#endif /* M_INT */

#ifndef HF_MAX_PARAMS
#define HF_MAX_PARAMS 10
#endif

#include <stdarg.h>
#include "mpltable.h"

#ifdef COMPLEX_STRUCT_REDEF
#undef COMPLEXF64
#define COMPLEXF64 COMPLEX_STRUCT_REDEF
#endif

#ifdef __cplusplus
     extern "C" {
#endif

/* DAG Conversion. These are implemented within Maple itself. */
/* The functions here have to be alphabetically ordered as in extsupp.c */
typedef struct _MKernelVectorDesc {
    /* Allocate Using Maple's Allocator */
    void   *(M_DECL *alloc) MPL_P(( M_INT nbytes ));
    ALGEB   (M_DECL *allocALGEB) MPL_P(( M_INT length, M_INT id ));

    /* Assign to Maple Variables */
    ALGEB   (M_DECL *assign) MPL_P(( ALGEB lhs, ALGEB rhs ));
    ALGEB   (M_DECL *assignIndexed) MPL_P(( ALGEB lhs, M_INT dim, 
					M_INT *ind, ALGEB rhs ));

    /* Free Memory Allocated By Maple's Allocator */
    void    (M_DECL *dispose) MPL_P(( ALGEB s ));

    /* Raising Errors */
    void    (M_DECL *error) MPL_P(( char *msg, /* ALGEB opt1, opt2, */ ... ));

    /* Evaluate an assigned name */
    ALGEB   (M_DECL *eval) MPL_P(( ALGEB s ));
    /* Evaluate a Maple Procedure or DAG using hardware floats */
    double  (M_DECL *evalhf) MPL_P(( ALGEB s ));
    /* first argument here is args[1] */
    double  (M_DECL *evalhfMapleProc) MPL_P(( ALGEB fn, int nargs, double *args ));

    /* Evaluate a Maple Procedure or statement */
    ALGEB   (M_DECL *evalMapleProc) MPL_P(( ALGEB fn, int nargs, /* ALGEB arg1, ALGEB arg2, */ ... ));
    ALGEB   (M_DECL *evalMapleProcedure) MPL_P(( ALGEB fn, ALGEB args ));
    ALGEB   (M_DECL *evalMapleStatement) MPL_P(( char *statement ));

    /* Data Queries */
    M_BOOL  (M_DECL *isAssignedName) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isComplexNumeric) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isInteger) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isInteger8) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isInteger16) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isInteger32) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isInteger64) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isMapleNULL) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isName) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isNumeric) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isPointer) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isPointerNULL) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isProcedure) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isRTable) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isString) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isTable) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isUnassignedName) MPL_P(( ALGEB s ));
    M_BOOL  (M_DECL *isUnnamedZero) MPL_P(( ALGEB s ));
    /* List Manipulation */
    ALGEB   (M_DECL *listAlloc) MPL_P(( M_INT num_members ));
    void    (M_DECL *listAssign) MPL_P(( ALGEB list, M_INT i, ALGEB val ));
    ALGEB   (M_DECL *listSelect) MPL_P(( ALGEB list, M_INT i ));

    /* Garbage Collection */
    void    (M_DECL *gcAllow) MPL_P(( ALGEB a ));
    void    (M_DECL *gcProtect) MPL_P(( ALGEB a ));

    /* Conversion From Maple Objects */
    COMPLEXF32     (M_DECL *mapleToComplexFloat32) MPL_P(( ALGEB s ));
    COMPLEXF64     (M_DECL *mapleToComplexFloat64) MPL_P(( ALGEB s ));
    CXDAG          (M_DECL *mapleToComplexFloatDAG) MPL_P(( ALGEB s ));
    FLOAT32        (M_DECL *mapleToFloat32) MPL_P(( ALGEB s ));
    FLOAT64        (M_DECL *mapleToFloat64) MPL_P(( ALGEB s ));
    INTEGER8       (M_DECL *mapleToInteger8) MPL_P(( ALGEB s ));
    INTEGER16      (M_DECL *mapleToInteger16) MPL_P(( ALGEB s ));
    INTEGER32      (M_DECL *mapleToInteger32) MPL_P(( ALGEB s ));
    INTEGER64      (M_DECL *mapleToInteger64) MPL_P(( ALGEB s ));
    M_BOOL	   (M_DECL *mapleToM_BOOL) MPL_P(( ALGEB s ));
    M_INT          (M_DECL *mapleToM_INT) MPL_P(( ALGEB s ));
    void*          (M_DECL *mapleToPointer) MPL_P(( ALGEB s ));
    char*          (M_DECL *mapleToString) MPL_P(( ALGEB s ));

    /* Determine the Number of Arguments in a Maple Object */
    M_INT   (M_DECL *numArgs) MPL_P(( ALGEB expr ));

    /* Rectangular Table (vector,matrix,array) Manipulation */
    void    (M_DECL *rtableAppendAttribute) MPL_P(( RTableSettings *s, char *name));
    void    (M_DECL *rtableAppendIndFn) MPL_P(( RTableSettings *s, ALGEB indfn ));
    RTableData (M_DECL *rtableAssign) MPL_P(( ALGEB rt, M_INT *index, RTableData val )); 
    ALGEB   (M_DECL *rtableCopy) MPL_P(( RTableSettings *s, ALGEB rt ));
    ALGEB   (M_DECL *rtableCopyImPart) MPL_P(( RTableSettings *s, ALGEB rt ));
    ALGEB   (M_DECL *rtableCopyRealPart) MPL_P(( RTableSettings *s, ALGEB rt ));
    ALGEB   (M_DECL *rtableCreate) MPL_P(( RTableSettings *s, void *pdata, M_INT *bounds ));
    void*   (M_DECL *rtableData) MPL_P(( ALGEB rt ));
    void    (M_DECL *rtableGetDefaults) MPL_P(( RTableSettings *s ));
    void    (M_DECL *rtableGetSettings) MPL_P(( RTableSettings *s, ALGEB rt ));
    M_INT   (M_DECL *rtableIndFn) MPL_P(( ALGEB rt, int num ));
    ALGEB   (M_DECL *rtableIndFnArgs) MPL_P(( ALGEB rt, int num ));
    M_BOOL  (M_DECL *rtableIsReal) MPL_P(( ALGEB rt ));
    M_INT   (M_DECL *rtableLowerBound) MPL_P(( ALGEB rt, int dim ));
    M_INT   (M_DECL *rtableNumElems) MPL_P(( ALGEB rt ));
    M_INT   (M_DECL *rtableNumDimensions) MPL_P(( ALGEB rt ));
    RTableData (M_DECL *rtableSelect) MPL_P(( ALGEB rt, M_INT *index )); 
    void    (M_DECL *rtableSetAttribute) MPL_P(( RTableSettings *s, char* name ));
    void    (M_DECL *rtableSetIndFn) MPL_P(( RTableSettings *s, ALGEB indfn ));
    void    (M_DECL *rtableSetType) MPL_P(( RTableSettings *s, int id, char* name));
    void    (M_DECL *rtableSparseCompact) MPL_P(( ALGEB rt ));
    NAG_INT* (M_DECL *rtableSparseIndexRow) MPL_P(( ALGEB rt, int dim ));
    ALGEB   (M_DECL *rtableSparseIndexSort) MPL_P(( ALGEB rt, int by_dim ));
    void    (M_DECL *rtableSparseSetNumElems) MPL_P(( ALGEB rt, M_INT num ));
    M_INT   (M_DECL *rtableSparseSize) MPL_P(( ALGEB rt ));
    M_INT   (M_DECL *rtableUpperBound) MPL_P(( ALGEB rt, int dim ));
    ALGEB   (M_DECL *rtableZipReIm) MPL_P(( RTableSettings *s, ALGEB rt_re, ALGEB rt_im ));

    /* Data Selection */
    ALGEB   (M_DECL *selectImaginaryPart) MPL_P(( ALGEB s ));
    ALGEB   (M_DECL *selectIndexed) MPL_P(( ALGEB s, M_INT dim, M_INT *ind ));
    ALGEB   (M_DECL *selectRealPart) MPL_P(( ALGEB s ));

    /* Data Uniquification */
    ALGEB   (M_DECL *simplify) MPL_P(( ALGEB s ));

    /* Table Manipulation */
    ALGEB   (M_DECL *tableAlloc) MPL_P(( void ));
    void    (M_DECL *tableAssign) MPL_P(( ALGEB table, ALGEB ind, ALGEB val ));
    void    (M_DECL *tableDelete) MPL_P(( ALGEB table, ALGEB ind ));
    M_BOOL  (M_DECL *tableHasEntry) MPL_P(( ALGEB table, ALGEB ind ));
    ALGEB   (M_DECL *tableSelect) MPL_P(( ALGEB table, ALGEB ind ));

    /* Conversion To Maple Objects */
    ALGEB   (M_DECL *toMapleBoolean) MPL_P(( long b ));
    ALGEB   (M_DECL *toMapleChar) MPL_P(( long c ));
    ALGEB   (M_DECL *toMapleComplex) MPL_P(( double re, double im ));
    ALGEB   (M_DECL *toMapleComplexFloat) MPL_P(( ALGEB re, ALGEB im ));
    ALGEB   (M_DECL *newMapleExpressionSequence) MPL_P(( int nargs ));
    ALGEB   (M_DECL *toMapleExpressionSequence) MPL_P(( int nargs, /* ALGEB arg1, ALGEB arg2, */ ... ));
    ALGEB   (M_DECL *toMapleInteger) MPL_P(( long i ));
    ALGEB   (M_DECL *toMapleInteger64) MPL_P(( INTEGER64 i ));
    ALGEB   (M_DECL *toMapleFloat) MPL_P(( double f ));
    ALGEB   (M_DECL *toMapleName) MPL_P(( char *n, M_BOOL is_global ));
    ALGEB   (M_DECL *toMapleNULL) MPL_P(( void ));
    ALGEB   (M_DECL *toMapleNULLPointer) MPL_P(( void ));
    ALGEB   (M_DECL *toMaplePointer) MPL_P(( void *v ));
    ALGEB   (M_DECL *toMapleRelation) MPL_P(( const char *const rel, ALGEB lhs, ALGEB rhs ));
    ALGEB   (M_DECL *toMapleString) MPL_P(( char *s ));
    ALGEB   (M_DECL *toMapleUneval) MPL_P(( ALGEB s ));

    /* Evaluate a C Function, Trapping Any Raised Errors */
    void*   (M_DECL *traperror) MPL_P(( void *(*proc) MPL_P(( void *data )),
                                 void *data, M_BOOL *errorflag ));

    /* Provide Run-time Information */
    void    (M_DECL *userinfo) MPL_P(( int level, char *name, char *msg ));

#ifndef ENHANCED_EF_BACKOUT

    /* Enhanced External Functions 
       (experimental and subject to change without notice) */
    ALGEB   (M_DECL *xxExtObHandleAlloc) MPL_P((void *val, void *typ,
                                            void (*marker)(void *v, void *t)
));
    M_BOOL  (M_DECL *xxExtObHandleTest)  MPL_P(( ALGEB ob ));
    void *  (M_DECL *xxExtObHandleType)  MPL_P(( ALGEB ob ));
    void *  (M_DECL *xxExtObHandleValue) MPL_P(( ALGEB ob ));

    void    (M_DECL *xxGcRequest) MPL_P(());
    M_BOOL  (M_DECL *xxGcAddEndListener)   MPL_P(( void (*fun) MPL_P(()) ));
    M_BOOL  (M_DECL *xxGcAddMarkListener)  MPL_P(( void (*fun) MPL_P(()) ));
    M_BOOL  (M_DECL *xxGcAddStartListener) MPL_P(( void (*fun) MPL_P(()) ));
    M_INT   (M_DECL *xxGcHeapObjectWalk) MPL_P(( void **splace, void **oplace,
					     M_INT obc, void **obv ));
    M_INT   (M_DECL *xxGcHeapSectionWalk)  MPL_P(( void **splace ));
    M_BOOL  (M_DECL *xxGcMarkOne) MPL_P(( ALGEB ));
    void ** (M_DECL *xxGcMemInfo) MPL_P(( ));

    ALGEB   (M_DECL *xxNewUnsimplified) MPL_P(( M_INT id, M_INT length ));
    M_INT   (M_DECL *xxObjectId)  MPL_P(( ALGEB ));
    M_INT   (M_DECL *xxObjectNArgs)  MPL_P(( ALGEB ));
    int     (M_DECL *printf) MPL_P(( const char *fmt, ... ));
#endif /*--ENHANCED_EF--*/

} MKernelVectorDesc, *MKernelVector;

#ifdef __cplusplus
     }
#endif

#endif /* MPLSHLIB_H */
