################################################################################
################################################################################
###
###	Filename: Edge.mpl
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
################################################################################
################################################################################
###
###	NAME
###
###		Edge -- directed edge constructor
###
###	SYNOPSIS
###
###		Edge( source:;anything, target::anything )::Edge
###
###	PARAMETERS
###
###		source -- the source vertex of the edge
###		target -- the target vertex of the edge
###
###	DESCRIPTION
###
###	The procedure `Edge()' is a constructor for directed edges in directed,
###	combinatorial graphs. Given two arbitary (non-NULL) Maple expressions,
###	SOURCE and TARGET, it returns an edge object that represents a directed
###	edge from SOURCE to TARGET. The edge object returned by this constructor
###	implements the `Edge' interface, also defined in this file, that requires
###	the exports
###
###		source, target, setsource, settarget.
###
###	These exports have the following semantics:
###
###		source -- (no argument) returns the source vertex of the edge
###		target -- (no argument) returns the target vertex of the edge
###		setsource  -- modifies the source vertex to point to its argument
###		settarget  -- modifies the target vertex to point to its argument
###
################################################################################
################################################################################

`type/Edge` := '`module`( source, target, setsource, settarget )':
#savelib( '`type/Edge`' ):
$ifdef	LOAD_REPOSITORY
savelib( '`type/Edge`' ):
$endif	# LOAD_REPOSITORY

Edge := proc( src, targ )
	description "edge constructor for combinatorial, directed graphs";
	option	remember; # ensure unicity

	module()
		description "a directed edge in a graph";
		local	the_source, the_target;
		export	source,		# return the source vertex
			target,		# return the target vertex
			setsource,	# modify the source vertex
			settarget;	# modify the target vertex

		#########
		# State #
		#########
		the_source := src;
		the_target := targ;

		#############
		# Behavior  #
		#############

		# Return the source vertex
		source := () -> the_source;

		# Return the target vertex
		target := () -> the_target;

		# Set the source vertex to V
		setsource := proc( v )
			the_source := v
		end proc;

		# Set the target vertex to V
		settarget := proc( v )
			the_target := v
		end proc;
	end module
end proc:

# Load-time assertions
e := Edge( 'a', 'b' ):
ASSERT( type( e, 'Edge' ) ):
ASSERT( e:-source() = a ):
ASSERT( e:-target() = b ):

#savelib( 'Edge' ):
$ifdef	LOAD_REPOSITORY
savelib( 'Edge' ):
$endif	# LOAD_REPOSITORY

################################################################################
################################################################################
###
###	End of File: Edge.mpl
###
################################################################################
################################################################################
