########################################################################
########################################################################
###
###	Filename: Graph1.mpl
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
########################################################################
########################################################################
###
###	NAME
###
###		Graph1 -- first graph constructor
###
###	SYNOPSIS
###
###		Graph1( e1, e2, e3, ... )::Graph
###
###	PARAMETERS
###
###		ei -- zero or more edges specified as two-member lists
###		      of the form [ source, target ].
###
###	DESCRIPTION
###
###	This procedure `Graph1()' is a constructor for directed graphs.
###	It returns a graph object, which satisfies the `Graph' interface
###	defined below as `type/Graph1`. The resulting graph object
###	has the exports
###
###		vertices, edges, addedge, order, size.
###
###	This have the following semantics.
###
###		vertices()::set -- returns the set of all vertices
###		edges()::set -- returns the set of edge objects
###		addedge( e )::NULL -- adds the edge pair [s,t] to the
###			edge set
###		order()::nonnegint -- returns the number of vertices
###		size()::nonnegint -- returns the number of edges
###
########################################################################
########################################################################

`type/Graph` := '`module`( order, size, edges, vertices, addedge )':
#savelib( '`type/Graph`' ):
$ifdef	LOAD_REPOSITORY
savelib( '`type/Graph`' ):
$endif	# LOAD_REPOSITORY

Graph1 := proc()
	description "directed graph constructor";
	local	vertex_set, edge_set;

	edge_set := { args };

	# Validate the arguments
	if map( type, edge_set, 'list' ) <> { true } then
		error "graph must be specified by a sequence of edges"
	end if;
	if map( nops, edge_set ) <> { 2 } then
		error "each edge must be specified as a [ source, target ] pair"
	end if;

	# Another way to do this
	#if map( type, edge_set, '[ anything, anything ]' ) then
	#	error "graph must be specified by its edges in the form "
	#		"[ source, target ]"
	#end if

	# Compute the set of vertices from the edge set
	vertex_set := map( op, edge_set );

	# Compute the set of edges by applying the edge
	# constructor to each vertex pair
	edge_set := map( Edge@op, edge_set );

	# Construct the graph object
	module()
		description "a directed graph";
		export	order,
			size,
			vertices,
			addedge,
			edges;

		vertices := () -> vertex_set;
		edges := () -> edge_set;
		order := () -> nops( vertices() );
		size := () -> nops( edges() );
		addedge := proc( src, targ )
			edge_set := { Edge( src, targ ) } union edge_set;
			vertex_set := { src, targ } union vertex_set;
			NULL
		end proc;
	end module
end proc:
#savelib( 'Graph1' ):
$ifdef	LOAD_REPOSITORY
savelib( 'Graph1' ):
$endif	# LOAD_REPOSITORY

########################################################################
########################################################################
###
###	End of File: Graph1.mpl
###
########################################################################
########################################################################
