########################################################################
########################################################################
###
###	Filename: vdeg.mpl
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
########################################################################
########################################################################
###
###	NAME
###
###		vdeg -- compute the in-degree and out-degree of a vertex
###
###	SYNOPSIS
###
###		vdeg( g::Graph, v::anything )::(nonnegint,nonnegint)
###
###	PARAMETERS
###
###		g -- a graph object
###		v -- any expression (non-NULL) representing a vertex
###
###	DESCRIPTION
###
###	The procedure `vdeg()' computes the in- and out-degrees of a
###	graph vertex V in a graph G. This procedure is ``generic'', in
###	that it works with graphs regardless of how they are represented
###	internally. The only requirement is that they satisfy the
###	`Graph' interface with the correct semantics.
###
###	Two graph constructors `Graph1()' and `Graph2()' are provided to
###	illustrate the representation independence of generic code. This
###	procedure `vdeg()' will work with graphs produced by either of
###	the two constructors.
###
########################################################################
########################################################################

$include	"Edge.mi"

vdeg := proc( G::Graph, v )
	description "generic procedure to compute the in- and out-degrees of a vertex";
	local	vs, vt;

	if member( v, G:-vertices() ) then
		vs := select( e -> evalb( v = Source( e ) ), G:-edges() );
		vt := select( e -> evalb( v = Target( e ) ), G:-edges() );
		nops( vs ), nops( vt )
	else
		0, 0
	end if
end proc:
#savelib( 'vdeg' ):
$ifdef	LOAD_LIBRARY
savelib( 'vdeg' ):
$endif	# LOAD_LIBRARY

########################################################################
########################################################################
###
###	End of File: vdeg.mpl
###
########################################################################
########################################################################
