#!/bin/sh
########################################################################
########################################################################
###
###	Filename: mload.sh
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
########################################################################
########################################################################
###
###	WARNING: Do not use this script until you are sure that your
###	standard Maple repository is write-protected.
###
########################################################################
########################################################################
###
###	Description: This is a simple shell program that you can use to
###	load the sample source code files into your private Maple
###	repository. It can be used on all UNIX platforms, and on
###	suitably equipped Windows machines.
###
###	How you invoke this program is platform dependent.
###
###	UNIX
###
###	To use this program, type (at the shell prompt)
###
###		./mload.sh FILE1 FILE2 ...
###
###	where FILE1, FILE2,..., are the Maple files that you want to
###	load into your repository. Alternatively, copy this program
###	to a directory searched by your shell and make it executable.
###	For example,
###
###		cp mload.sh $HOME/bin/mload
###		chmod 0755 $HOME/bin/mload
###
###	Then you can call this script like any other program, such as
###
###		mload FILE1 FILE2 ...
###
###	(Depending upon which shell you use, you may have to issue the
###	shell command
###
###		rehash
###
###	before `mload' will be recognised.) If you are a system
###	administrator, you may want to make this script available
###	to all your users by copying it to some public directory
###	(e.g., `/usr/local/bin').
###
###	WINDOWS
###
###	To use this program on Windows (95,98,NT), you will need to
###	have installed a POSIX shell (several are available). Then,
###	from that shell you can invoke this script the same way that
###	you can on UNIX platforms. From the DOS prompt, you can type
###
###		sh mload.sh FILE1 FILE2 ...
###
###	assuming that your shell is called `sh'. (If not, replace `sh'
###	by the actual name in the command line above.)
###
###	MAC
###
###	This program does not work on Macintosh platforms.
###
###	This program uses the environment variable `MAPLELIB'. It should
###	be set to the name of the directory in which your private Maple
###	repository resides. If this environment variable is not set,
###	the default location is
###
###		${HOME}/maple/lib
###
###	If the directory does not contain a Maple repository, then
###	loading these files will create a number of files with a
###	`.m' suffix. Some files will fail to load if you do not have
###	the corresponding subdirectory structure. For example, a name
###	like `type/foo`, to be loaded in the absence of a repository,
###	requires that there be a subdirectory called `type'. Saving
###	that name will create the file `$HOME/maple/lib/type/foo.m'.
###
###	This program should not be used unless your standard Maple
###	repository is write-protected. You can make your repository
###	unwritable by issuing the shell command
###
###		chmod a-w maple.lib maple.ind maple.rep
###
###	in the directory where those files reside.
###
###	This script should work with any POSIX shell. If your POSIX
###	shell is called ksh or bash (for example), change the first
###	line ("#!/bin/sh") of this file to reflect the actual location
###	of your shell.
###
########################################################################
########################################################################

me="`basename $0`";

MAPLELIB=${MAPLELIB:-${HOME}/maple/lib}
export MAPLELIB

MAPLE='maple -s -q -a';

for FILE in $*; do

	# Check that the file exists and is readable
	if test ! -r "${FILE}"; then
		exec 1>&2;
		echo "${me}: *** file \"${FILE}\" does not exist or is not readable";
		exit 1
	fi;

	# Check for a directory.
	if test -d "${FILE}"; then
		echo "${me}: warning: \"${FILE}\" is a directory -- skipping";
	fi;

	# Load the file
	${MAPLE} -D LOAD_REPOSITORY < "${FILE}";
	rc=$?;
	case ${rc} in
		0)	:;; # okay
		*)
			exec 1>&2;
			echo "${me}: *** cannot load file \"${FILE}\"";
			exit ${rc};
			;;
	esac;
done;

exit 0;
########################################################################
########################################################################
###
###	End of File: mload.sh
###
########################################################################
########################################################################
