########################################################################
########################################################################
###
###	Filename: circle.mpl
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
########################################################################
########################################################################
###
###	Description: This file defines the `circle' submodule of the
###	Shapes:-Shapes submodule in the `Shapes' package. It contains
###	the circle-specific subroutines necessary to implement the
###	package.
###
###	In addition to the required subroutines `make', `area', and
###	`circumference', circles support routines for computing the
###	radius, diameter, and center point.
###
###	Circles are represented by unevaluated function calls of the
###	form
###
###		CIRCLE( center, radius )
###
###	in which `center' is the center point of the circle, and `radius'
###	is its radius.
###
########################################################################
########################################################################

circle := module()
	export	make,
		center,
		centre,
		radius,
		area,
		circumference;
	local	diameter;

	# This is the `make' subroutine for circles. It returns an
	# unevaluated function call that represents a circle with a
	# given center and radius.
	make := proc( cntrPt, radius )
		'CIRCLE'( cntrPt, radius )
	end proc;

	# The center of a circle can be extracted using this syntax
	# procedure. (It is customary to provide both British and
	# American spelling.) No real computation is involved, since
	# the center of a circle is part of its instance data.
	center := circ -> op( 1, circ );
	centre := center; # UK spelling

	# Another syntax procedure is provided to query a circle
	# shape for its radius. Because the radius is stored as
	# instance data for the circle, no real work has to be done.
	radius := circ -> op( 2, circ );

	# The diameter of a circle is computed by this local (private)
	# subroutine.
	diameter := circ -> 2 * radius( circ );

	# We use the local subroutine `diameter' to compute the
	# circumference of a circle. This is called by the top-level
	# `circumference' routine in the `Shapes' package via the
	# dispatch table in the file `shapes.mpl'.
	circumference := circ -> Pi * diameter( circ );

	# The area of a circle is computed by this procedure. This
	# procedure is called from the top-level `area' export of the
	# `Shapes' package via the dispatching mechanism described in
	# the file `shapes.mpl', and in the Programming Guide.
	area := circ -> Pi * radius( circ )^2;
end module:

########################################################################
########################################################################
###
###	End of File: circle.mpl
###
########################################################################
########################################################################
