########################################################################
########################################################################
###
###	Filename: point.mpl
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
########################################################################
########################################################################
###
###	Description: This file defines the submodule Shape:-point of the
###	Shapes module that implements the Shapes package. Its fully
###	qualified name is :-Shapes:-Shapes:-point.
###
###	This file is included into the file `shapes.mpl' via the Maple
###	preprocesor. The shapes.mpl defines the package; it is not used
###	on its own. However, in the case of this particular example, it
###	*is* possible to load this submodule as a top-level module in
###	its own right. That is because, in this case, the `point'
###	submodule makes no reference to lexically scoped variables in
###	any parent module. (This will not generally be the case.)
###
########################################################################
########################################################################

# This is the module definition for the `point' submodule. It is, via
# file inclusion, a triply nested submodule of the Shapes package. When
# this file is included in `shapes.mpl', this assignment refers to the
# exported member `point' of the module :-Shapes:-Shapes.
point := module()
	description "support routines for point shapes";
	export	make,
		xcoord,
		ycoord,
		area,
		circumference;

	# This is the `make' subroutine for constructing points.
	# It is called via the dispatch mechanism described in the
	# file `shapes.mpl' for the top-level `make' export. In this
	# submodule, the name `make' refers to the exported variable
	# for this submodule, shadowing the top-level `make' variable
	# in the `:-Shapes' module. If required, the `make' export of
	# `:-Shapes' could be referenced within this submodule by
	# using the syntax `:-Shapes:-make'.

	# The `make' procedure for points requires the "x" and "y"
	# coordinates of the point to be constructed. The point is
	# represented as an unevaluated function call, as shown
	# here. The mnemonic "tag" `POINT' is used by some of the
	# dispatch mechanisms illustrated in the file `shapes.mpl'.
	make := ( x, y ) -> 'POINT'( x, y );

	# Computing the area of a point is trivial. This implementation
	# simply ignores its arguments and returns the value 0. This
	# design offers speed at the cost of robustness. No argument
	# checking overhead is incurred, but neither are any diagnostic
	# issues when the procedure is called with an incorrect argument.
	area := () -> 0;

	# This is the subroutine used for computing the circumference
	# of a point. It is called indirectly through the table defined
	# in the top-level `:-Shapes' module.
	circumference := () -> 0;

	# These are two syntax procedures for extracting the parts of
	# a point. A point consists of its "x" and "y" coordinates;
	# the following two routines provide access to those data. The
	# advantage of providing these syntax procedures is that the
	# client of this package need not be concerned with the actual
	# representation of points.
	xcoord := p -> op( 1, p );
	ycoord := p -> op( 2, p );
end module:

# No call to `savelib()' appears in this file, because it is intended
# to be loaded as part of the `Shapes' package via file inclusion. The
# contents of this submodule are saved to the repository along with the
# rest of the `Shapes' package.

########################################################################
########################################################################
###
###	End of File: point.mpl
###
########################################################################
########################################################################
