########################################################################
########################################################################
###
###	Filename: square.mpl
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
########################################################################
########################################################################
###
###	Description: This file contains the definition of the `square'
###	submodule of the `Shapes' package.
###
###	In addition to the `area' and `circumference' subroutines that
###	are required of all shapes, squares support a `side_length'
###	routine that computes the length of a side.
###
###	Squares are represented by unevaluated function calls of the
###	form
###
###		SQUARE( center, side )
###
###	in which `center' is a point representing the center of the
###	square, and `side' is the length of a side of the square.
###
###	This file is included into the top-level source file `shapes.mpl'
###	via the Maple preprocessor.
###
########################################################################
########################################################################

# This is the definition of the `square' submodule of the module
# Shapes:-Shapes. It defines a submodule because it is included via
# the preprocessor at a point in the file `shapes.mpl' where its
# definition makes it so.
square := module()
	export	make,		# constructor for squares
		side_length,	# returns the length of a side
		area,		# returns the area of a square
		circumference;	# returns the circumference of a square
	description "support routines for squares";

	# This is the `make' constructor for squares. It is called
	# via the dispatch mechanism described in the file `shapes.mpl',
	# and in the Programming Guide. The value returned is an
	# expression that represents a square by storing, as the
	# arguments to an unevaluated function call, the center point
	# and side length of the square.
	make := proc( cntr, side )
		'SQUARE'( cntr, side )
	end proc;

	# This is merely a syntax procedure for squares. It returns the
	# length of one side of the square given as its sole argument.
	side_length := sq -> op( 2, sq );

	# The area of a square with side length S is computed by the
	# formula S^2. This routine is called from the top-level `area'
	# procedure exported from the `Shapes' package.
	area := sq -> side_length( sq )^2;

	# The circumference of a square is the sum of the length of its
	# sides or, equivalently, four times the length of one side.
	# This procedure is called via the table dispatch method shown
	# in the file `shapes.mpl'.
	circumference := sq -> 4 * side_length( sq );
end module:

# No call to `savelib()' appears in this file, since it is not intended
# to be used on its own. It is only included into the `shapes.mpl' file.

########################################################################
########################################################################
###
###	End of File: square.mpl
###
########################################################################
########################################################################
