########################################################################
########################################################################
###
###	Filename: triangle.mpl
###
###     Copyright (c) 1999 Waterloo Maple, Inc.
###
###     THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF WMI
###     The copyright notice above does not evidence any
###     actual or intended publication of such source code.
###
########################################################################
########################################################################
###
###	Description: This file implements the `triangle' submodule of
###	the `Shapes' package. It contains triangle-specific subroutines
###	for the package.
###
###	In addition to the required `make', `area', and `circumference'
###	subroutines, triangles also support a subroutine `side' of two
###	arguments. The first is the triangle on which to operate. The
###	second is one of the integers 1, 2, and 3, used to select a
###	specific side of the triangle, whose length is returned.
###
###	Triangles are represented by the ordered collection of their
###	sides, as segments.
###
########################################################################
########################################################################

triangle := module()
	export	make,
		area,
		side,
		circumference;

	# This is the `make' subroutine for triangles. It takes three
	# points as arguments--the vertices of the triangle--and returns
	# an unevaluated function call of the form
	#
	#	TRIANGLE( side1, side2, side3 )
	#
	# which represents the triangle.
	# For illustrative purposes, both the segment-specific
	# constructor `segment:-make', and the top-level shape constructor
	# `:-Shapes:-make' are shown here.
	# Observe the three different names spelled "make" in this entire
	# definition. One is the export `make' in this submodule. The 
	# second is the `make' exported from the `segment' submodule 
	# (which must be addressed as shown, since the `make' local to 
	# this module shadows it). The third is the top-level name `make',
	# referenced here as :-Shapes:-make. The ":-" prefix is also 
	# required here; that is, "Shapes:-make" is not a valid reference.
	# This is because, in this submodule, the name `Shapes' refers to 
	# the submodule `:-Shapes:-Shapes', which does not export the name
	# `make'.
	make := proc( pt1, pt2, pt3 )
		'TRIANGLE'(
			segment:-make( pt1, pt2 ),
			:-Shapes:-make( ':-segment', pt2, pt3 ),
			segment:-make( pt3, pt1 ) )
	end proc;

	# The length of a specific side can be computed by calling
	# this procedure with a triangle and one of the number 1,2,3
	# as the index into its list of sides.
	side := proc( tri, idx )
		if not member( idx, { 1, 2, 3 } ) then
			error "triangles have only three sides"
		end if;
		op( idx, tri )
	end proc;

	# This is the `circumference' subroutine for triangles.
	circumference := tri -> add( segment:-len( side( tri, i ) ), i = 1 .. 3 );

	area := proc( tri )
		local	base, height;
		base := segment:-len( side( tri, 1 ) );
		height := segment:-len( segment:-make( segment:-midpoint( side( tri, 1 ) ),
				segment:-end_point( side( tri, 2 ) ) ) );
		base * height / 2;
	end proc;
end module:

########################################################################
########################################################################
###
###	End of File: triangle.mpl
###
########################################################################
########################################################################
